@@ -34,7 +34,7 @@ InputImpl::InputImpl (const string& option_key)
 {
 }
 
-void InputImpl::Parse (const string& expr, const ptime& timestamp, bool verbose, Database::CollectionElements& elements) const throw (Xception)
+void InputImpl::Parse (const string& expr, const ptime& timestamp, bool verbose, Database::CollectionsItems& items) const throw (Xception)
 {
 	string line(expr);
 
@@ -83,15 +83,24 @@ void InputImpl::Parse (const string& expr, const ptime& timestamp, bool verbose,
 	}
 
 	if (verbose)
-		cout << (format("    [%s] %s") % lexical_cast<string,ptime>(t) % expr).str() << endl;
+	{
+		// we want to have possible date and time specifications
+		// from the original expression only once on a line, so
+		// we have to format the output a bit
+		string s(line);
+		trim(s);
+		if (!comment.empty())
+			s += string(" ; ") + any_cast<string>(comment);
+		cout << (format("    [%s] %s") % lexical_cast<string, ptime>(t) % s).str() << endl;
+	}
 
 	// now build a data vector for a database insert
 	if (!comment.empty())
-		elements.push_back(Database::CollectionElement("#", t, comment));
+		items.insert(Database::CollectionsItems::value_type("#", Database::CollectionItem(t, comment)));
 
 	while (regex_search(line, mr, rx_ratio, boost::match_not_dot_newline))
 	{
-		elements.push_back(Database::CollectionElement(mr[2], t, Ratio(lexical_cast<double>(mr[1]), lexical_cast<double>(mr[3]))));
+		items.insert(Database::CollectionsItems::value_type(mr[2], Database::CollectionItem(t, Ratio(lexical_cast<double>(mr[1]), lexical_cast<double>(mr[3])))));
 		line.erase(mr.position(), mr.length());
 	}
 
@@ -100,7 +109,7 @@ void InputImpl::Parse (const string& expr, const ptime& timestamp, bool verbose,
 		string name(mr[2]);
 		if (name.empty())
 			name = '*';
-		elements.push_back(Database::CollectionElement(name, t, lexical_cast<double>(mr[1])));
+		items.insert(Database::CollectionsItems::value_type(name, Database::CollectionItem(t, lexical_cast<double>(mr[1]))));
 		line.erase(mr.position(), mr.length());
 	}
 
@@ -176,8 +185,8 @@ void FileInputImpl::Do (const Config& config, Database& database) const throw (X
 	{
 		try
 		{
-			// collect collection elements
-			Database::CollectionElements elements;
+			// get collections items
+			Database::CollectionsItems items;
 			{
 				ifstream ifs(filename.c_str(), ios::in);
 				if (!ifs.good())
@@ -187,12 +196,12 @@ void FileInputImpl::Do (const Config& config, Database& database) const throw (X
 				if (verbose)
 					cout << "parsing " << m_vector_key << " file " << filename << endl;
 
-				ProcessFile(config, database, ifs, elements);
+				ProcessFile(config, database, ifs, items);
 			}
 
 			// add the contents of the file in a single
 			// transaction to the database
-			database.AddCollectionElements(elements);
+			database.AddCollectionsItems(items);
 
 			// cleanup
 			if (!keep)
